// ===========================================================================
// 	GSApp.c 
// ===========================================================================


// ===========================================================================
//	Includes
// ===========================================================================

#include "GameScene_prefix.h"

#include <stdlib.h>

#if (kQAPlatform == kQAMacOS)
	#include <Memory.h>
	#include <Windows.h>
	#include <Displays.h>
	#include <QuickDraw.h>
#endif

#if (kQAPlatform == kQAWin32)
#include "resource.h"
#endif

#include "GSGameScene.h"
#include "GSRandomTriMesh.h"
#include "GSUtilities.h"
#include "GSError.h"


// ===========================================================================
//	Constants
// ===========================================================================

#define						kWindowPercentage				0.75
#define						kVendorID_Apple					0
#define						kEngineID_AppleHardware			(-1)


// ===========================================================================
//	Globals
// ===========================================================================

#if (kQAPlatform == kQAMacOS)
WindowPtr					gWindowPtr = nil;
#elif (kQAPlatform == kQAWin32)
HWND						gWindowPtr = NULL;
#endif

TQADevice					gDevice;
TQARect						gContentRect;
TGSTestInfo					gTestInfo;

#if (kQAPlatform == kQAWin32)

HINSTANCE hAppInst;          	// current instance

char szAppName[] = "RAVE Demo";		// The name of this application
BOOL		isQuitting = false, isSpecialAlphaTest = false;

#endif /* kQAWin32 */

// ===========================================================================
//	Private Prototypes
// ===========================================================================

#if (kQAPlatform == kQAMacOS)
	void  GSInitializeOperatingSystem(void);
#elif (kQAPlatform == kQAWin32)
	BOOL InitApplication(HINSTANCE hInstance);

	LRESULT CALLBACK WndProc(
		HWND hWnd,        
		UINT message,      
		WPARAM uParam,     
		LPARAM lParam);
#endif

#if (kQAplatform == kQAMacOS)
	TGSError GSCreateWindow(void);
#elif (kQAPlatform == kQAWin32)
	TGSError GSCreateWindow(HINSTANCE hInstance);
#endif

void GSClearWindow(void);

TGSError GSSetupTestInfo(void);

TGSError GSCreateDrawContext(
	Boolean					inUsingAppleHardware);

void GSWaitForExit(void);


// ===========================================================================
//	main
// ===========================================================================
#if (kQAPlatform == kQAMacOS)
	int
main(void)
#elif (kQAPlatform == kQAWin32)
	int CALLBACK WinMain(
			HINSTANCE hInstance,
			HINSTANCE hPrevInstance,
			LPSTR lpCmdLine,
			int nCmdShow)
#endif
{
	TGSError				gsError;
	
		// this call enables Apple's QuickDraw3D Hardware Accelerator.  since
		// this card is not a fully-compliant RAVE engine, it must be explicitly
		// enabled by any app that wants to use it.  otherwise, it will never
		// be returned by calls to QAGetFirstEngine().  the Apple HW Accelerator
		// has a number of limitations which must be handled by any app that
		// makes RAVE calls to it.  we handle these limitations by checking
		// which engine is being used when setting up the TGSTestInfo and when
		// creating the draw context.
//	QAEngineEnable(kVendorID_Apple, kEngineID_AppleHardware);
	
#if (kQAPlatform == kQAMacOS)
	GSInitializeOperatingSystem();
#endif

#if (kQAPlatform == kQAWin32)
    MSG msg;
    HANDLE hAccelTable;

    if (!hPrevInstance) 
	{     
		if (!InitApplication(hInstance)) 
			return (FALSE);    
    }
    hAppInst = hInstance; 

    hAccelTable = LoadAccelerators (hInstance, MAKEINTRESOURCE(IDR_GENERIC));

	gsError = GSCreateWindow(hInstance);
	
	if (gsError != kGSError_None)
		exit(EXIT_FAILURE);
	
	GSClearWindow();
	
	gsError = GSSetupTestInfo();
	
	if (gsError != kGSError_None)
		exit(EXIT_FAILURE);

	while( 1 )
	{
	    if( PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) )
	    {
	        if( !GetMessage( &msg, NULL, 0, 0 ) )
	        {
	            return msg.wParam;
	        }
	        TranslateMessage(&msg);
	        DispatchMessage(&msg);
	    }
	    else
	    {
	        /* chew up cycles here */
	    }
	}
    return (msg.wParam); // Returns the value from PostQuitMessage

#elif (kQAPlatform == kQAMacOS)	

	gsError = GSCreateWindow();
	
	if (gsError != kGSError_None)
		exit(EXIT_FAILURE);
	
	GSClearWindow();
	
	gsError = GSSetupTestInfo();
	
	if (gsError != kGSError_None)
		exit(EXIT_FAILURE);

	gsError = GSDrawGameScene(&gTestInfo);

	if (gsError != kGSError_None) {
		exit(EXIT_FAILURE);
	}
	
	GSWaitForExit();
	return (0);
#endif /* kQAMacOS */

}


#if (kQAPlatform == kQAMacOS)
// ===========================================================================
//	GSInitializeOperatingSystem
// ===========================================================================
	void 
GSInitializeOperatingSystem(void)
{
	MaxApplZone();
	
	MoreMasters(); 
	MoreMasters(); 
	MoreMasters(); 
	
	InitGraf(&qd.thePort);
	InitWindows();
	InitCursor();

	FlushEvents(everyEvent, 0);
}
#endif /* kQAMacOS */

#if (kQAPlatform == kQAWin32)

BOOL InitApplication(HINSTANCE hInstance)
{
    WNDCLASS  wc;

    wc.style         = CS_HREDRAW | CS_VREDRAW |
						CS_OWNDC; /* important, because we need to keep referencing
									 the same handle for updates! */
    wc.lpfnWndProc   = (WNDPROC)WndProc;       
    wc.cbClsExtra    = 0;                      
    wc.cbWndExtra    = 0;                     
    wc.hInstance     = hInstance;             
    wc.hIcon         = LoadIcon (hInstance, MAKEINTRESOURCE(IDI_APP)); 
    wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
    wc.lpszMenuName  = MAKEINTRESOURCE(IDR_GENERIC); 
    wc.lpszClassName = szAppName;              

    return (RegisterClass(&wc));
}

LRESULT CALLBACK WndProc(
	HWND hWnd,        
	UINT message,      
	WPARAM uParam,     
	LPARAM lParam)   
{
    int			wmId, wmEvent;
	PAINTSTRUCT PaintStruct;

    switch (message) 
	{
        case WM_COMMAND:

            wmId    = LOWORD(uParam);
            wmEvent = HIWORD(uParam);
            switch (wmId) 
			{
                case IDM_EXIT:
                    DestroyWindow (hWnd);
                    break;
                default:
                    return (DefWindowProc(hWnd, message, uParam, lParam));
            }
            break;

		case WM_CREATE:
			break;

        case WM_DESTROY:  // message: window being destroyed
            PostQuitMessage(0);
            break;

		case WM_LBUTTONDOWN:
		{
			/*
			if (DoAnotherTest() == -1)
				DestroyWindow(hWnd);
			*/
			TQAError	gsError;
			BOOL		aSetResult;

			if (isQuitting == true)
			{
				DestroyWindow(hWnd);
				break;
			}
			switch (gTestInfo.mBitmapFormat)
			{
				case kQAPixel_Alpha1:
					aSetResult = SetWindowText(hWnd, (LPCTSTR)"GameSceneTestA1");
					break;
				case kQAPixel_RGB16:
					aSetResult = SetWindowText(hWnd, (LPCTSTR)"GameSceneTest16");
					break;
				case kQAPixel_ARGB16:
					aSetResult = SetWindowText(hWnd, (LPCTSTR)"GameSceneTestA16");
					break;
				case kQAPixel_RGB32:
					aSetResult = SetWindowText(hWnd, (LPCTSTR)"GameSceneTest32");
					break;
				case kQAPixel_ARGB32:
					aSetResult = SetWindowText(hWnd, (LPCTSTR)"GameSceneTestA32");
					break;
				case kQAPixel_CL4:
					aSetResult = SetWindowText(hWnd, (LPCTSTR)"GameSceneTestCL4");
					break;
				case kQAPixel_CL8:
					aSetResult = SetWindowText(hWnd, (LPCTSTR)"GameSceneTestCL8");
					break;
			}
			gsError = GSDrawGameScene(&gTestInfo);

			if (isSpecialAlphaTest == true)
				isQuitting = true;
			else
			{
				if (gTestInfo.mBitmapFormat < kQAPixel_CL8)
					gTestInfo.mBitmapFormat++;
				else
				{
					isSpecialAlphaTest = true;
					gTestInfo.mBitmapFormat = kQAPixel_Alpha1;
				}

				if (gTestInfo.mTextureFormat < kQAPixel_CL8)
					gTestInfo.mTextureFormat++;
				else
				{
					isSpecialAlphaTest = true;
					gTestInfo.mTextureFormat = kQAPixel_RGB32;
				}
			}
			break;
		}

		case WM_PAINT:
			BeginPaint(hWnd, &PaintStruct);
			/* TBD: Put up a message saying to click to begin each test */
			EndPaint(hWnd, &PaintStruct);
			break;

        default:          // Passes it on if unproccessed
                return (DefWindowProc(hWnd, message, uParam, lParam));
    }
    return (0);
}

#endif /* kQAWin32 */

// ===========================================================================
//	GSCreateWindow
// ===========================================================================
#if (kQAPlatform == kQAMacOS)
	TGSError GSCreateWindow(void)
#elif (kQAPlatform == kQAWin32)
	TGSError GSCreateWindow(HINSTANCE hInstance)
#endif
{
#if (kQAPlatform == kQAMacOS)
	GDHandle				deepestGDevice = nil;
	Rect					displayRect, windowRect;
	long					displayWidth, displayHeight, windowSideLength;

		// find the screen with the greatest number of colors
	deepestGDevice = GSFindDeepestMacDisplay();
	GSAssert_(deepestGDevice);
	
		// since this is Mac code, we use a GDevice as the RAVE device
	gDevice.deviceType = kQADeviceGDevice;
	gDevice.device.gDevice = deepestGDevice;
	
	displayRect = (**deepestGDevice).gdRect;
	
	displayWidth = displayRect.right - displayRect.left;
	displayHeight = displayRect.bottom - displayRect.top;
	
		// size the window according to the size of the display
	windowSideLength = (displayHeight < displayWidth) ? displayHeight : displayWidth;
	windowSideLength = (long) (windowSideLength * kWindowPercentage);
	
	windowRect.left = displayRect.left + ((displayWidth - windowSideLength) / 2);
	windowRect.top = displayRect.top + ((displayHeight - windowSideLength) / 2);
	
	windowRect.right = windowRect.left + windowSideLength;
	windowRect.bottom = windowRect.top + windowSideLength;
	
		// the window rect is in global coordinates, but the TQARect which is 
		// used to create a draw context must be in device-local coordinates,
		// so calculate the window's position based on the upper-left corner 
		// of the display it is on.
	gTestInfo.mContextRect.left = windowRect.left - displayRect.left;
	gTestInfo.mContextRect.right = windowRect.right - displayRect.left;
	gTestInfo.mContextRect.top = windowRect.top - displayRect.top;
	gTestInfo.mContextRect.bottom = windowRect.bottom - displayRect.top;

		// create a new color window, with no grow, zoom or close boxes
	gWindowPtr = NewCWindow(nil, &windowRect, "\pGame Scene", true, noGrowDocProc,
			nil, false, nil);
		
	if (gWindowPtr == nil)
		return kGSError_OS;

#elif (kQAPlatform == kQAWin32)
	{
		BOOL	aResult;
		RECT	aRect;
		int		width, height;

		gWindowPtr = CreateWindow(
				szAppName,          
				"Game Scene",             
				WS_OVERLAPPEDWINDOW | WS_VISIBLE, 
				50, 50, 400, 400, 	 // x, y, width, height
				NULL,                
				NULL,                
				hInstance,           
				NULL                 
		);
		if (gWindowPtr == nil)
			return kGSError_OS;
		gDevice.deviceType = kQADeviceWin32DC;
		gDevice.device.hdc = GetDC(gWindowPtr);

			/* fix up client rect to insure it is square */
		aResult = GetClientRect(gWindowPtr, &aRect);
		width = aRect.right - aRect.left;
		height = aRect.bottom - aRect.top;
  		if (width > height)
			aRect.right = aRect.left + height;
		else
			aRect.bottom = aRect.top + width;

		gTestInfo.mContextRect.left = aRect.left;
		gTestInfo.mContextRect.right = aRect.right;
		gTestInfo.mContextRect.top = aRect.top;
		gTestInfo.mContextRect.bottom = aRect.bottom;
	}

#endif /* kQAMacOS || kQAWin32 */
	
	return kGSError_None;
}


// ===========================================================================
//	GSClearWindow
// ===========================================================================
	void
GSClearWindow(void)
{
#if (kQAPlatform == kQAMacOS)
	GrafPtr					savedPort;
	
		// remember the current port
	GetPort(&savedPort);
	
		// erase the content area of our window
	SetPort(gWindowPtr);
	EraseRect(&gWindowPtr->portRect);
	
		// reset the previous port
	SetPort(savedPort);
#endif
}


// ===========================================================================
//	GSSetupTestInfo
// ===========================================================================
	TGSError
GSSetupTestInfo(void)
{
	TGSError				gsError;
	TQAError				qaError;
	long					vendorID = 0, engineID = 0;
	Boolean					usingAppleHardware = false;
	
		// find the first engine which is best suited to the device we'll be
		// drawing on
	gTestInfo.mEngine = QADeviceGetFirstEngine(&gDevice);
	
	if (gTestInfo.mEngine == nil) {
		return kGSError_RAVE;
	}
	
		// use the TQADevice we set up in GSCreateWindow
	gTestInfo.mDevice = gDevice;
	
		// you can change the format of the textures by uncommenting a different
		// line below.  note that if you choose CL4 or CL8, you must set 
		// mTextureMakeMipMap to false, because the GSPicture object cannot
		// currently mipmap color-lookup table textures.  also, GSPicture does 
		// not do a good job converting 32-bit PICTs with alpha channels 
		// into CL4 or CL8 textures.
	gTestInfo.mTextureFormat = kQAPixel_RGB16;
//	gTestInfo.mTextureFormat = kQAPixel_ARGB16;
//	gTestInfo.mTextureFormat = kQAPixel_RGB32;
//	gTestInfo.mTextureFormat = kQAPixel_ARGB32;
//	gTestInfo.mTextureFormat = kQAPixel_CL4;
//	gTestInfo.mTextureFormat = kQAPixel_CL8;
	
		// the textures will not be locked or mipmapped
	gTestInfo.mTextureFlags = kQATexture_None;
	
		// we do not want them to be detached or mipmapped
	gTestInfo.mTextureDetach = false;
	gTestInfo.mTextureMakeMipMap = false;
	
		// none of the textures we are using have any transparency
	gTestInfo.mTextureMakeBlackTransparent = false;
	gTestInfo.mTextureUseTransparentIndex = false;
	
		// you can change the format of the bitmaps by uncommenting a different
		// line below.  note that if you use RGB16 or RGB32, the transparent
		// areas of the bitmaps will be opaque, since RAVE ignores the alpha
		// values in those two modes.  also, GSPicture does not do a good job
		// converting 32-bit PICTs with alpha channels into CL4 or CL8 bitmaps.
//	gTestInfo.mBitmapFormat = kQAPixel_Alpha1;
	gTestInfo.mBitmapFormat = kQAPixel_RGB16;
//	gTestInfo.mBitmapFormat = kQAPixel_ARGB16;
//	gTestInfo.mBitmapFormat = kQAPixel_RGB32;
//	gTestInfo.mBitmapFormat = kQAPixel_ARGB32;
//	gTestInfo.mBitmapFormat = kQAPixel_CL4;
//	gTestInfo.mBitmapFormat = kQAPixel_CL8;

		// the bitmaps will not be locked or detached
	gTestInfo.mBitmapFlags = kQABitmap_None;
	gTestInfo.mBitmapDetach = false;
	
		// we want the black background of the bitmaps to be transparent if
		// we're using one of the ARGB formats, and we want the first color in
		// the color table to be transparent if we're using a CL format.
	gTestInfo.mBitmapMakeBlackTransparent = true;
	gTestInfo.mBitmapUseTransparentIndex = true;
	
		// determine which engine we're using
	qaError = QAEngineGestalt(gTestInfo.mEngine, kQAGestalt_VendorID, &vendorID);
	
	if (qaError != kQANoErr) {
		return kGSError_RAVE;
	}
	
	qaError = QAEngineGestalt(gTestInfo.mEngine, kQAGestalt_EngineID, &engineID);
	
	if (qaError != kQANoErr) {
		return kGSError_RAVE;
	}
	
		// we're interested in special-casing only Apple's first hardware card
	usingAppleHardware = (vendorID == kVendorID_Apple && engineID == kEngineID_AppleHardware);
	
	if (usingAppleHardware) {
			// we are using the Apple Hardware Accelerator, which is not a
			// RAVE-compliant engine.  therefore we have to change a few things
			// to handle its idiosyncracies.
			//
			// the Apple HW Accelerator requires that all textures submitted to 
			// it be mipmapped.  if you want the textures to be in kQAPixel_CL8 
			// or kQAPixel_CL4 format, then you must set this field to false, 
			// since the TGSImage object currently cannot mipmap textures in 
			// color table format.  this means you cannot use color table 
			// textures with the Apple HW Accelerator (which currently cannot 
			// handle them anyway).
		gTestInfo.mTextureMakeMipMap = true;
		
			// the previous line will cause the TGSImage object to be mipmapped.
			// the next line contains the flags which are passed to QATextureNew
			// so that the call will know the textures have been mipmapped.
		gTestInfo.mTextureFlags |= kQATexture_Mipmap;
		
			// the Apple HW also requires that all textures be in RGB32 format
		gTestInfo.mTextureFormat = kQAPixel_RGB32;
		
			// the only bitmap format that the Apple HW can handle is Alpha1
		gTestInfo.mBitmapFormat = kQAPixel_Alpha1;
	}
	
	gsError = GSCreateDrawContext(usingAppleHardware);
	
	if (gsError != kGSError_None) {
		return gsError;
	}
	
	return kGSError_None;
}


// ===========================================================================
//	GSCreateDrawContext
// ===========================================================================
	TGSError
GSCreateDrawContext(
	Boolean					inUsingAppleHardware)
{
	TQAError				qaError;
	
	gTestInfo.mContextFlags = kQAContext_None;
	
		// you can make the draw context be double-buffered by uncommenting the
		// following line.  when a context is double-buffered, you won't see
		// anything until the entire scene is drawn, at which point everything
		// will appear.
//	gTestInfo.mContextFlags |= kQAContext_DoubleBuffer;
	
	if (inUsingAppleHardware) {
			// Apple's HW Accelerator requires all draw contexts to have the 
			// kQAContext_DeepZ flag set
		gTestInfo.mContextFlags |= kQAContext_DeepZ;
	}
	
	qaError = QADrawContextNew(&gTestInfo.mDevice, &gTestInfo.mContextRect, nil,
			gTestInfo.mEngine, gTestInfo.mContextFlags, &gTestInfo.mContext);
			
	if (qaError != kQANoErr) {
		return kGSError_RAVE;
	}
	
		// once you've created the draw context, there are a number of state
		// variables which can be set, such as the background color, turning
		// antialiasing on and off, etc.  below we set some of these if we
		// are using the Apple HW Accelerator.
	if (inUsingAppleHardware) {
			// we turn on the Perspective Z flag so that Apple's HW will
			// perspective-correct the textures.  the Apple Software drawing 
			// engine always does perspective-correction.
		QASetInt(gTestInfo.mContext, kQATag_PerspectiveZ, kQAPerspectiveZ_On);
		
			// Apple's HW also requires the optional texture mapping operation
			// state variable to be set a certain way
		QASetInt(gTestInfo.mContext, kQATag_TextureOp, 
				kQATextureOp_Modulate & kQATextureOp_Highlight);
	}
	
	return kGSError_None;
}


// ===========================================================================
//	GSWaitForExit
// ===========================================================================
	void
GSWaitForExit(void)
{
#if (kQAPlatform == kQAMacOS)
	while (!Button())
		;
		
	FlushEvents(everyEvent, 0);
#endif
}


