/******************************************************************************
 **																			 **
 ** 	Module:		QAL_Test.c												 **
 ** 																		 **
 ** 	Purpose: 	Main file for QAL test app								 **
 ** 																		 **
 ** 	Author:		Mike W. Kelley											 **
 **																			 **
 **					2/3/95	Revised for 0.9 SDK release						 **
 ** 																		 **
 ** 	Copyright (C) 1994-95 Apple Computer, Inc.  All rights reserved.	 **
 ** 																		 **
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "RAVE.h"

/* Macintosh */
#if (kQAPlatform == kQAMacOS)
	#include <Types.h>
	#include <QuickDraw.h>
	#include <Windows.h>
#elif (kQAPlatform == kQAWin32)
	#include <windows.h>
	#include "resource.h"		// Windows resource IDs
#endif

/* Project */
#include "QAL_Test.h"

/*******************************************************************************************
 *
 * Defines that affect how the tests are run.
 *
 ******************************************************************************************/

#if (kQAPlatform == kQAMacOS)
	#define dWaitForButton	1		/* If defined, wait for button click between tests */
									/* On Win32, tests are triggered by WM_LBUTTONDOWN */
#endif

/*******************************************************************************************
 *
 * Include TtQAL.h to get the declaration of TtRegister(), which we need to call
 * directly when running linked in.
 *
 ******************************************************************************************/

#ifdef dLinkEngine
	#include "TtQAL.h"
#endif

/*
	Forward declarations
*/
#if (kQAPlatform == kQAWin32)
BOOL InitApplication(HINSTANCE hInstance);

BOOL InitInstance(
        HINSTANCE       hInstance,
        int             nCmdShow);

LRESULT CALLBACK WndProc(
	HWND hWnd,        
	UINT message,      
	WPARAM uParam,     
	LPARAM lParam);

#endif /* kQAWin32 */

 /*******************************************************************************************
 *
 * Globals.
 *
 ******************************************************************************************/

long			gWindowWidth, gWindowHeight;
TQAEngine		*gEngine [kQAL_Windows];
TQADrawContext	*gDrawContext [kQAL_Windows];
TQATexture		*gTexture [kQAL_Windows];

long			tinselTest = 0, tinselStyle = 0;

#if (kQAPlatform == kQAWin32)

HINSTANCE hAppInst;          	// current instance

char szAppName[] = "RAVE Demo";		// The name of this application

#endif /* kQAWin32 */
/*******************************************************************************************
 *
 * Static variables used by this file's functions only.
 *
 ******************************************************************************************/

static float frand (void)
{
	float	result;
#if (kQAPlatform == kQAMacOS)
	result = (float)((Random() + 32768) / 65536.0f); /* -32K >= Random <= 32K */
#else
	result = (float)((rand() * 2.0f) / 65536.0f);	/* 0 >= rand <= 32K */
#endif
	return result;
}

/*******************************************************************************************
 *
 * Main. This runs the tests, clearing the windows before each test and waiting for
 * a mouse click after each test. Each test is run four ways: Single and double buffered,
 * with and without 2D clipping.
 *
 ******************************************************************************************/

#if (kQAPlatform == kQAMacOS)
	int main()
#elif (kQAPlatform == kQAWin32)
	int CALLBACK WinMain(
			HINSTANCE hInstance,
			HINSTANCE hPrevInstance,
			LPSTR lpCmdLine,
			int nCmdShow)
#endif
{	
	/* If we are running with a shared library, TdLinkEngine should not be defined! */
	#ifdef dLinkEngine
		(void) TtRegister();
	#endif

#if (kQAPlatform == kQAMacOS)
	InitToolbox ();
#endif

#if (kQAPlatform == kQAWin32)
    MSG msg;
    HANDLE hAccelTable;

    if (!hPrevInstance) 
	{     
		if (!InitApplication(hInstance)) 
			return (FALSE);    
    }
    hAppInst = hInstance; 

    hAccelTable = LoadAccelerators (hInstance, MAKEINTRESOURCE(IDR_GENERIC));

	if ( ! CreateTestWindows (kQAL_MaxWindowSize, kQAL_MaxWindowSize, hInstance, nCmdShow))
		return (FALSE);
	if ( ! CreateWindowClip ())
		return (FALSE);
	if ( ! TestTextureNew())
		return (FALSE);

	while( 1 )
	{
	    if( PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) )
	    {
	        if( !GetMessage( &msg, NULL, 0, 0 ) )
	        {
	            return msg.wParam;
	        }
	        TranslateMessage(&msg);
	        DispatchMessage(&msg);
	    }
	    else
	    {
	        /* chew up cycles here */
	    }
	}
    return (msg.wParam); // Returns the value from PostQuitMessage

#elif (kQAPlatform == kQAMacOS)	

	if ( ! CreateTestWindows (kQAL_MaxWindowSize, kQAL_MaxWindowSize))
		return (-1);
	if ( ! CreateWindowClip ())
		return (-1);
	if ( ! TestTextureNew())
		return (-1);

	do
	{
		aResult = DoAnotherTest();
	#ifdef dWaitForButton
		/* Wait for a mouse button click.. */
		
		while ( ! Button())
			/* Do nothing */ ;
		FlushEvents (everyEvent, 0);
	#endif
	} while (aResult != -1);

	return (0);
#endif /* kQAMacOS */
}


	/* Returns -1 if no more tests */
int		DoAnotherTest(void)
{
	long		i;
	TQAVGouraud	vGouraud[3];
	TQAVTexture	vTexture[3];
	long		flushCount;
	int			aResult = 0;

	if (tinselTest == 0)
	{
		switch (tinselStyle)
		{
			case kStyleSingleBuffered:
				if ( ! TestDrawContextNew (kQAContext_DeepZ, FALSE))
					return (-1);
				break;
			
			case kStyleDoubleBuffered:
				if ( ! TestDrawContextNew (kQAContext_DeepZ | kQAContext_DoubleBuffer, FALSE))
					return (-1);
				break;
				
			case kStyleSingleBufferedClip:
				if ( ! TestDrawContextNew (kQAContext_DeepZ, TRUE))
					return (-1);
				break;
			
			case kStyleDoubleBufferedClip:
				if ( ! TestDrawContextNew (kQAContext_DeepZ | kQAContext_DoubleBuffer, TRUE))
					return (-1);
				break;
		}
	}
	ClearWindows();	
	switch (tinselTest)
	{
		case kTestPoints:
			TestRenderStart();
			for (i = 0, flushCount = kNPoints / kNFlushes; i++ < kNPoints; /* Nothing */)
			{
				TestRandomVGouraud (&vGouraud[0]);
				TestDrawPoint (&vGouraud[0]);
				if ((i % flushCount) == 0)
					TestFlush();
			}
			TestRenderEnd();
			break;
	
		case kTestLines:
			TestRenderStart();
			for (i = 0, flushCount = kNLines / kNFlushes; i++ < kNLines; /* Nothing */)
			{
				TestRandomVGouraud (&vGouraud[0]);
				TestRandomVGouraud (&vGouraud[1]);
				TestDrawLine (&vGouraud[0], &vGouraud[1]);
				if ((i % flushCount) == 0)
					TestFlush();
			}
			TestRenderEnd();
			break;

		case kTestTriGouraud:
			TestRenderStart();
			for (i = 0, flushCount = kNTris / kNFlushes; i++ < kNTris; /* Nothing */)
			{
				TestRandomVGouraud (&vGouraud[0]);
				TestRandomVGouraud (&vGouraud[1]);
				TestRandomVGouraud (&vGouraud[2]);
				
				TestDrawTriGouraud (&vGouraud[0],
						&vGouraud[1], &vGouraud[2], kQATriFlags_None);
				
				if ((i % flushCount) == 0)
					TestFlush();
			}
			TestRenderEnd();
			break;

		case kTestTriTexture:
			TestRenderStart();
			TestSetStateTexture();
			for (i = 0, flushCount = kNTris / kNFlushes; i++ < kNTris; /* Nothing */)
			{
				/* Setup the u,v values before calling TestRandomVTexture(),
				 * as it references them. */
				vTexture[0].uOverW = 0.0f;
				vTexture[0].vOverW = 0.0f;

				vTexture[1].uOverW = 0.0f;
				vTexture[1].vOverW = 1.0f;

				vTexture[2].uOverW = 1.0f;
				vTexture[2].vOverW = 1.0f;

				TestRandomVTexture (&vTexture[0]);
				TestRandomVTexture (&vTexture[1]);
				TestRandomVTexture (&vTexture[2]);
				
				TestDrawTriTexture (&vTexture[0],
						&vTexture[1], &vTexture[2], kQATriFlags_None);
				
				if ((i % flushCount) == 0)
					TestFlush();
			}
			TestRenderEnd();
			break;
	} /* switch tinselTest */
	tinselTest++;
	if (tinselTest > kMaxTest)
	{
		tinselTest = 0;
		TestDrawContextDelete();
		tinselStyle++;
		if (tinselStyle > kMaxStyle)
			aResult = -1;
	}
	return aResult;
}

#if (kQAPlatform == kQAWin32)

BOOL InitApplication(HINSTANCE hInstance)
{
    WNDCLASS  wc;

    wc.style         = CS_HREDRAW | CS_VREDRAW |
						CS_OWNDC; /* important, because we need to keep referencing
									 the same handle for updates! */
    wc.lpfnWndProc   = (WNDPROC)WndProc;       
    wc.cbClsExtra    = 0;                      
    wc.cbWndExtra    = 0;                     
    wc.hInstance     = hInstance;             
    wc.hIcon         = LoadIcon (hInstance, MAKEINTRESOURCE(IDI_APP)); 
    wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
    wc.lpszMenuName  = MAKEINTRESOURCE(IDR_GENERIC); 
    wc.lpszClassName = szAppName;              

    return (RegisterClass(&wc));
}

LRESULT CALLBACK WndProc(
	HWND hWnd,        
	UINT message,      
	WPARAM uParam,     
	LPARAM lParam)   
{
    int			wmId, wmEvent;
	PAINTSTRUCT PaintStruct;

    switch (message) 
	{
        case WM_COMMAND:

            wmId    = LOWORD(uParam);
            wmEvent = HIWORD(uParam);
            switch (wmId) 
			{
                case IDM_ABOUT:
                    break;
				case IDM_OPEN:
					break;
				case IDM_SAVEAS:
					break;
                case IDM_EXIT:
                    DestroyWindow (hWnd);
                    break;
                default:
                    return (DefWindowProc(hWnd, message, uParam, lParam));
            }
            break;

		case WM_CREATE:
			break;

        case WM_DESTROY:  // message: window being destroyed
            PostQuitMessage(0);
            break;

		case WM_LBUTTONDOWN:
			if (DoAnotherTest() == -1)
				DestroyWindow(hWnd);
			break;

		case WM_PAINT:
			BeginPaint(hWnd, &PaintStruct);
			/* TBD: Put up a message saying to click to begin each test */
		/*	DoAnotherTest(); */				/* this does first test */
			EndPaint(hWnd, &PaintStruct);
			break;

        default:          // Passes it on if unproccessed
                return (DefWindowProc(hWnd, message, uParam, lParam));
    }
    return (0);
}

#endif /* kQAWin32 */

/*******************************************************************************************
 *
 * Set xyzw fields of a TQAVGouraud randomly. All other fields get simple defaults.
 *
 ******************************************************************************************/

void TestRandomVGouraud (
	TQAVGouraud		*v)
{
	v->x = frand() * gWindowWidth;
	v->y = frand() * gWindowHeight;
	v->z = frand();
	v->invW = 1.0f / v->z;
	
	v->a = 1.0f;
	v->r = frand();
	v->g = frand();
	v->b = frand();
}

/*******************************************************************************************
 *
 * Set xyzw fields of a TQAVTexture randomly. The incoming u,v values are multiplied
 * by the random invW value; all other fields get simple defaults.
 *
 ******************************************************************************************/

void TestRandomVTexture (
	TQAVTexture		*v)
{
	v->x = frand() * gWindowWidth;
	v->y = frand() * gWindowHeight;
	v->z = frand();
	v->invW = 1.0f / v->z;
	
	v->uOverW *= v->invW;
	v->vOverW *= v->invW;
	
	v->a = 1.0f;
	v->r = 0.0f;
	v->g = 0.0f;
	v->b = 0.0f;
	
	v->kd_r = 1.0f;
	v->kd_g = 1.0f;
	v->kd_b = 1.0f;
	
	v->ks_r = 0.0f;
	v->ks_g = 0.0f;
	v->ks_b = 0.0f;
}

/*******************************************************************************************
 *
 * Miscellaneous test functions.
 *
 ******************************************************************************************/

void TestRenderStart (void)
{
	long		i;
	
	for (i = 0; i < kQAL_Windows; ++i)
	{
		if (gDrawContext [i])
		{
			QARenderStart (gDrawContext [i], NULL, NULL);
		}
	}
}

void TestRenderEnd (void)
{
	long		i;
	TQAError	status;
	
	for (i = 0; i < kQAL_Windows; ++i)
	{
		if (gDrawContext [i])
		{
			status = QARenderEnd (gDrawContext [i], NULL);
		}
	}
}

void TestSetStateTexture (void)
{
	long		i;
	
	for (i = 0; i < kQAL_Windows; ++i)
	{
		if (gDrawContext [i])
		{
			QASetPtr (gDrawContext [i], kQATag_Texture, gTexture [i]);
			
			/*
			 * NOTE: The current Apple software rasterizer always assumes these
			 * kQATextureOp_Modulate | kQATextureOp_Highlight are set, as that's
			 * the standard QuickDraw 3D mode. So if you change this, you may
			 * find that the software rasterizer isn't a good test case.
			 * (This will be fixed soon...)
			 */
			
			QASetInt (gDrawContext [i], kQATag_TextureOp,
					kQATextureOp_Modulate | kQATextureOp_Highlight);
		}
	}
}

void TestFlush (void)
{
	long		i;
	
	for (i = 0; i < kQAL_Windows; ++i)
	{
		if (gDrawContext [i])
		{
			QAFlush (gDrawContext [i]);
		}
	}
}

void TestDrawPoint (
	const TQAVGouraud	*v)
{
	long		i;
	
	for (i = 0; i < kQAL_Windows; ++i)
	{
		if (gDrawContext [i])
		{
			QADrawPoint (gDrawContext [i], v);
		}
	}
}

void TestDrawLine (
	const TQAVGouraud	*v0,
	const TQAVGouraud	*v1)
{
	long		i;
	
	for (i = 0; i < kQAL_Windows; ++i)
	{
		if (gDrawContext [i])
		{
			QADrawLine (gDrawContext [i], v0, v1);
		}
	}
}

void TestDrawTriGouraud (
	const TQAVGouraud	*v0,
	const TQAVGouraud	*v1,
	const TQAVGouraud	*v2,
	unsigned long		flags)
{
	long		i;
	
	for (i = 0; i < kQAL_Windows; ++i)
	{
		if (gDrawContext [i])
		{
			QADrawTriGouraud (gDrawContext [i], v0, v1, v2, flags);
		}
	}
}

void TestDrawTriTexture (
	const TQAVTexture	*v0,
	const TQAVTexture	*v1,
	const TQAVTexture	*v2,
	unsigned long		flags)
{
	long		i;
	
	for (i = 0; i < kQAL_Windows; ++i)
	{
		/*
		 * Test both for a draw context, and that a texture was created.
		 */
		
		if (gDrawContext [i] && gTexture [i])
		{
			QADrawTriTexture (gDrawContext [i], v0, v1, v2, flags);
		}
	}
}

long TestTextureNew (void)
{
	TexturePixel		*pixmap;
	long				x, y;
	long				i;
	
	/*
	 * Create the source pixmap. Note that we can't free it (unless we
	 * called QATextureDetach()). We just do a procedural map with some
	 * checkerboard effect...
	 */
	
	if ( ! (pixmap = (TexturePixel *) malloc (sizeof (*pixmap) * kTextureSize * kTextureSize)))
	{
		return (FALSE);
	}
	
	for (x = 0; x < kTextureSize; ++x)
	{
		for (y = 0; y < kTextureSize; ++y)
		{
			pixmap [x + (y << kTextureSizeNBits)].a = 0xff;
			pixmap [x + (y << kTextureSizeNBits)].r = (x & 0x1f) << 3;
			pixmap [x + (y << kTextureSizeNBits)].g = (y & 0x1f) << 3;
			pixmap [x + (y << kTextureSizeNBits)].b = (((x + y) / 2) & 0x1f) << 3;
		}
	}

	for (i = 0; i < kQAL_Windows; ++i)
	{
		if (gEngine [i])
		{
			/*
			 * Create the texture. If this fails, gTexture[i] will be NULL.
			 * This is ok, as TestDrawTriTexture() always tests it before calling
			 * QADrawTriTexture().
			 */
			
			gTexture [i] = CreateMipMapTexture (
					gEngine [i],								/* store context */
					pixmap,										/* high res pix map */
					kTextureSize * sizeof (*pixmap),			/* row bytes */
					kTextureSizeNBits,							/* widthNBits */
					kTextureSizeNBits);							/* heightNBits */
		}
	}
	
	return (TRUE);
}















